@@ -54,9 +54,10 @@ static int midi_send_command_with_response __P((struct midi_softc *softc,
 static int midi_uart __P((struct midi_softc *));
 static int midi_enter_smpte_mode __P((struct midi_softc *));
 static int midi_event2smf __P((struct midi_softc *, struct event *,
-    struct stynamic *));
+    struct stynamic *, int));
 static int midi_next_byte __P((struct midi_softc *, void *));
-static int midi_uio2event __P((struct midi_softc *, void *, struct event *));
+static int midi_uio2event __P((struct midi_softc *, void *, struct event *,
+    int));
 static int midi_fix2var __P((u_long, u_char *));
 static int midi_var2fix __P((u_char *, u_long *));
 static u_long midi_smf2timing_tick __P((struct midi_softc *, long));
@@ -64,6 +65,7 @@ static u_long midi_timing2smf_tick __P((struct midi_softc *, long, long));
 static int midi_midi2SMPTE __P((struct event *, struct SMPTE_frame *));
 static int midi_SMPTE_framerate __P((struct SMPTE_frame *, int /* u_char */));
 static void midi_timeout __P((TIMEOUT_ARG arg));
+static void midi_write_timeout __P((TIMEOUT_ARG arg));
 static void midi_add_complete_event __P((struct midi_softc *));
 static void midi_schedule_timeout __P((struct midi_softc *, u_long));
 static void midi_write_event __P((struct midi_softc *, struct event *, int,
@@ -109,6 +111,11 @@ midi_init_dev(softc, addr)
 	softc->status |= MIDI_THRU;
 #endif
 
+	softc->raw_count = 0;
+	softc->timed_count = 0;
+	softc->oread_count = 0;
+	softc->owrite_count = 0;
+
 	rev = 0x01;
 	if (!midi_reset(softc)) {
 		if (!midi_reset(softc)) {
@@ -148,11 +155,12 @@ gen_midiopen(dev, flags, pgid, client)
 	void *client;
 {
 	register struct midi_softc *softc;
-	register int smpte, unit;
+	register int smpte, raw, unit;
 
 	unit = GET_MINOR(dev);
 	smpte = unit & 0x80;
-	unit &= 0x7f;
+	raw = unit & 0x40;
+	unit &= 0x3f;
 
 	softc = UNIT_TO_SOFTC(unit, client);
 	if (softc == NULL)
@@ -160,105 +168,131 @@ gen_midiopen(dev, flags, pgid, client)
 
 	if (softc->status & MIDI_UNAVAILABLE)
 		return (EIO);
-	
-	if (softc->status & MIDI_OPEN)
+
+	/*
+	 * must do special voodoo if already open for reading and
+	 * now opening for timed use.
+	 */
+	if (!raw && softc->timed_count != 0)
 		return (EBUSY);
+
+	if (!raw)
+		softc->pgid = pgid;
+
+	/* check to see if this is the first open or additional */
+	if (softc->raw_count + softc->timed_count == 0) {
+		softc->status = MIDI_OPEN | (softc->status & MIDI_THRU);
+		/*
+		 * reads will block until something appears
+		 */
+		softc->status |= MIDI_RD_BLOCK;
+
+		/* initialize the queues */
+		midi_initq(softc->rqueue);
+		midi_initq(softc->wqueue);
+
+		softc->partial_event.event.len = 0;
+		softc->partial_event.event.datad = NULL;
+
+		/* make sure we are in UART mode */
+		if (!midi_fullreset(softc)) {
+			LOGERR("Couldn't put MPU401 into UART mode!\n");
+			softc->status |= MIDI_UNAVAILABLE;
+			return (EIO);
+		}
+
+		/* if high bit is set, open device in SMPTE mode */
+		if (!smpte)
+			softc->status &= ~MIDI_EXTCLK;
+		else {
+			if (!(softc->features & SMPTE_EQUIP))
+				return (ENXIO);
+			/* reset again so we can issue commands */
+			(void)midi_reset(softc);
+			if (!midi_enter_smpte_mode(softc)) {
+				LOGERR("Couldn't enter SMPTE mode\n");
+				return (EIO);
+			}
+			if (!midi_uart(softc)) {
+				/* GJW maybe figure out which MQ board it is..*/
+				LOGERR("Couldn't put MQX-32M into UART mode\n");
+				return (EIO);
+			}
+			softc->status |= MIDI_EXTCLK;
+		}
+
+		/* XXX Do this the right way! */
+		/*
+		 * GJW i.e. change midiintr() so we can read 0xf1's but
+		 * ignore everything else when in SMPTE play mode.. ??
+		 */
+		if (softc->status & MIDI_EXTCLK)
+			softc->status |= MIDI_READING;
+
+		softc->status |= MIDI_FIRST_WRITE;
+		softc->status |= MIDI_FIRST_SMPTE;
+	}
+
+	if (raw)
+		softc->raw_count++;
 	else
-		softc->status = MIDI_OPEN;
-#ifdef DEFAULT_THRU
-	softc->status |= MIDI_THRU;
-#endif
+		softc->timed_count++;
 
-	/*
-	 * reads will block until something appears
-	 */
-	softc->status |= MIDI_RD_BLOCK;
-
-	/* initialize the queues */
-	midi_initq(softc->rqueue);
-	midi_initq(softc->wqueue);
-
-	softc->partial_event.event.len = 0;
-	softc->partial_event.event.datad = NULL;
-
-	/* make sure we are in UART mode */
-	if (!midi_fullreset(softc)) {
-		LOGERR("Couldn't put MPU401 into UART mode!\n");
-		softc->status |= MIDI_UNAVAILABLE;
-		return (EIO);
-	}
-
-	/* if high bit is set, open device in SMPTE mode */
-	if (!smpte)
-		softc->status &= ~MIDI_EXTCLK;
-	else {
-		if (!(softc->features & SMPTE_EQUIP))
-			return (ENXIO);
-		/* reset again so we can issue commands */
-		(void)midi_reset(softc);
-		if (!midi_enter_smpte_mode(softc)) {
-			LOGERR("Couldn't enter SMPTE mode\n");
-			return (EIO);
-		}
-		if (!midi_uart(softc)) {
-			/* GJW maybe figure out which MQ board it is..*/
-			LOGERR("Couldn't put MQX-32M into UART mode\n");
-			return (EIO);
-		}
-		softc->status |= MIDI_EXTCLK;
-	}
-		
-
-	softc->pgid = pgid;
-
-	/* are we going to read, write or both? */
-	if (flags & MIDI_OREAD)
+	if (flags & MIDI_OREAD) {
+		softc->oread_count++;
 		softc->status |= MIDI_READING;
-	else
-		softc->status &= ~MIDI_READING;
-	if (flags & MIDI_OWRITE)
+	}
+	if (flags & MIDI_OWRITE) {
+		softc->owrite_count++;
 		softc->status |= MIDI_WRITING;
-	else
-		softc->status &= ~MIDI_WRITING;
+	}
 
-	/* XXX Do this the right way! */
-	/*
-	 * GJW i.e. change midiintr() so we can read 0xf1's but
-	 * ignore everything else when in SMPTE play mode.. ??
-	 */
-	if (softc->status & MIDI_EXTCLK)
-		softc->status |= MIDI_READING;
-
-	softc->status |= MIDI_FIRST_WRITE;
-	softc->status |= MIDI_FIRST_SMPTE;
 	return (0);
 }
 
 int
-gen_midiclose(softc)
+gen_midiclose(softc, flags)
 	struct midi_softc *softc;
+	int flags;
 {
+	int raw;
 
-	/*
-	 * we're not going to finish closing until everything has
-	 * been played.
-	 */
-	if (softc->status & MIDI_WRITING) {
-		/* same as MFLUSHQ ioctl */
-		if (softc->wqueue->count != 0) {
-			softc->status |= MIDI_FLUSH_SLEEP;
-			SLEEP_INTR(&softc->sleep_flush, "midiclose");
-			softc->status &= ~MIDI_FLUSH_SLEEP;
-		}
+	raw = flags & MIDI_RAW_DEVICE;
+
+	if (raw)
+		softc->raw_count--;
+	else
+		softc->timed_count--;
+
+	if (flags & MIDI_OREAD) {
+		if (--softc->oread_count == 0)
+			softc->status &= ~MIDI_READING;
+	}
+	if (flags & MIDI_OWRITE) {
+		if (--softc->owrite_count == 0)
+			softc->status &= ~MIDI_WRITING;
 	}
 
-	/* turn off any notes that might be stuck on */
-	midi_reset_devices(softc);
-	midi_fullreset(softc);
+	if (softc->raw_count + softc->timed_count == 0) {
+		/*
+		 * we're not going to finish closing until everything has
+		 * been played.
+		 */
+		if (softc->status & MIDI_WRITING) {
+			/* same as MFLUSHQ ioctl */
+			if (softc->wqueue->count != 0) {
+				softc->status |= MIDI_FLUSH_SLEEP;
+				SLEEP_INTR(&softc->sleep_flush, "midiclose");
+				softc->status &= ~MIDI_FLUSH_SLEEP;
+			}
+		}
 
-	softc->status &= ~MIDI_READING;
-	softc->status &= ~MIDI_WRITING;
-	softc->status &= ~MIDI_OPEN;
+		/* turn off any notes that might be stuck on */
+		midi_reset_devices(softc);
+		midi_fullreset(softc);
+
+		softc->status &= ~MIDI_OPEN;
+	}
 	return (0);
 }
 
@@ -269,16 +303,23 @@ gen_midiread(softc, uio, flags)
 	int flags;
 {
 	struct event *event;
-	int error, init_len, num_to_move;
+	int error, init_len, num_to_move, raw;
 
+	raw = flags & MIDI_RAW_DEVICE;
 	init_len = UIO_REMAIN(uio);
 	if (softc->rqueue->count == 0 && softc->rpartial.len == 0) {
 		if (flags & MIDI_NDELAY)
 			return (EWOULDBLOCK);
 		else {
-			softc->status |= MIDI_RD_SLEEP;
-			SLEEP_INTR(&softc->sleep_read, "midiread");
-			softc->status &= ~MIDI_RD_SLEEP;
+			do {
+				softc->status |= MIDI_RD_SLEEP;
+				error = SLEEP_INTR(&softc->sleep_read,
+				    "midiread");
+				softc->status &= ~MIDI_RD_SLEEP;
+			} while (error == 0 && softc->rqueue->count == 0 &&
+			    softc->rpartial.len == 0);
+			if (error)
+				return (error);
 		}
 	}
 	while (UIO_REMAIN(uio)) {
@@ -290,18 +331,18 @@ gen_midiread(softc, uio, flags)
 				softc->status |= MIDI_RD_BLOCK;
 				return (0);
 			}
-			midi_event2smf(softc, event, &softc->rpartial);
+			midi_event2smf(softc, event, &softc->rpartial, raw);
 			stynamic_release(&event->data);
 		}
 		/* read out as much of rpartial as possible */
 		num_to_move = MIN(softc->rpartial.len, UIO_REMAIN(uio));
 		if (softc->rpartial.len <= STYNAMIC_SIZE) {
 			if ((error = MOVE_TO_UIO(uio, softc->rpartial.datas,
-			    num_to_move)) == -1)
+			    num_to_move)) != 0)
 				return (error);
 		} else {
 			if ((error = MOVE_TO_UIO(uio, softc->rpartial.datad,
-			    num_to_move)) == -1)
+			    num_to_move)) != 0)
 				return (error);
 		}
 		stynamic_shift(&softc->rpartial, num_to_move);
@@ -316,27 +357,33 @@ gen_midiwrite(softc, uio, flags)
 	int flags;
 {
 	struct event event, *e;
-	int convert;
+	int convert, error, raw;
 
+	raw = flags & MIDI_RAW_DEVICE;
 	/* check to see if we'll block */
-	if (softc->status & MIDI_WR_BLOCK) {
+	if (!raw && (softc->status & MIDI_WR_BLOCK)) {
 		if (flags & MIDI_NDELAY)
 			return (EWOULDBLOCK);
 		else {
-			softc->status |= MIDI_WR_SLEEP;
-			SLEEP_INTR(&softc->sleep_write, "midiwrite");
-			softc->status &= ~MIDI_WR_SLEEP;
+			do {
+				softc->status |= MIDI_WR_SLEEP;
+				error = SLEEP_INTR(&softc->sleep_write,
+				    "midiwrite");
+				softc->status &= ~MIDI_WR_SLEEP;
+			} while (error == 0 && softc->status & MIDI_WR_BLOCK);
+			if (error != 0)
+				return (error);
 		}
 	}
 	/* if returns from sleep and should abort because of DRAIN */
-	if (softc->status & MIDI_WR_ABORT) {
+	if (!raw && (softc->status & MIDI_WR_ABORT)) {
 		softc->status &= ~MIDI_WR_ABORT;
 		return (0);
 	}
 
 	stynamic_init(&event.data);
 	while (UIO_REMAIN(uio)) {
-		if (softc->wqueue->count == MIDI_Q_SIZE) {
+		if (!raw && (softc->wqueue->count == MIDI_Q_SIZE)) {
 			softc->status |= MIDI_WR_BLOCK;
 			if (!(softc->status & MIDI_TIMEOUT_PENDING)) {
 				if (!midi_peekq(softc->wqueue, &e))
@@ -346,9 +393,15 @@ gen_midiwrite(softc, uio, flags)
 			if (flags & MIDI_NDELAY)
 				return (0);
 			else {
-				softc->status |= MIDI_WR_SLEEP;
-				SLEEP_INTR(&softc->sleep_write, "midiwrite");
-				softc->status &= ~MIDI_WR_SLEEP;
+				do {
+					softc->status |= MIDI_WR_SLEEP;
+					error = SLEEP_INTR(&softc->sleep_write,
+					    "midiwrite");
+					softc->status &= ~MIDI_WR_SLEEP;
+				} while (error == 0 &&
+				    softc->wqueue->count == MIDI_Q_SIZE);
+				if (error != 0)
+					return (error);
 			}
 		}
 
@@ -357,13 +410,13 @@ gen_midiwrite(softc, uio, flags)
 		 * 2) convert it from SMF to board format
 		 * 3) deal with it
 		 */
-		convert = midi_uio2event(softc, uio, &event);
+		convert = midi_uio2event(softc, uio, &event, raw);
 		switch (convert) {
 		case 0:
 			break;
 		case -1:
 			/* not a complete event - we'll get it next time */
-			if (!(softc->status & MIDI_TIMEOUT_PENDING)) {
+			if (!raw && !(softc->status & MIDI_TIMEOUT_PENDING)) {
 				if (!midi_peekq(softc->wqueue, &e))
 					return (0);
 				midi_schedule_timeout(softc, e->time);
@@ -373,21 +426,28 @@ gen_midiwrite(softc, uio, flags)
 			return (convert);
 		}
 
-		if (midi_enq(softc->wqueue, &event) == -1)
-			return (EIO);
+		/* 0 delay timeout for writing the event */
+		if (raw) {
+			midi_copy_event(&softc->write_event, &event);
+			TIMEOUT(&softc->write_timeout_id, midi_write_timeout,
+			    softc, 0);
+		} else {
+			if (midi_enq(softc->wqueue, &event) == -1)
+				return (EIO);
+		}
 		stynamic_release(&event.data);
 		/* set flag so next time we cross LOW water we will SIGIO */
-		if (softc->wqueue->count >= MIDI_LOW_WATER)
+		if (!raw && (softc->wqueue->count >= MIDI_LOW_WATER))
 			softc->status |= MIDI_SENDIO;
 	}
 	/*
 	 * we set the flag if the last write filled the queue, but
 	 * we don't need to block
 	 */
-	if (softc->wqueue->count == MIDI_Q_SIZE)
+	if (!raw && (softc->wqueue->count == MIDI_Q_SIZE))
 		softc->status |= MIDI_WR_BLOCK;
 
-	if (!(softc->status & MIDI_TIMEOUT_PENDING)) {
+	if (!raw && !(softc->status & MIDI_TIMEOUT_PENDING)) {
 		if (!midi_peekq(softc->wqueue, &e))
 			return (0);
 		midi_schedule_timeout(softc, e->time);
@@ -762,6 +822,7 @@ midi_fullreset(softc)
 
 	/* clear running play state */
 	softc->writers = 0;
+	softc->raw_writers = 0;
 	softc->readrs = 0;
 
 	/* reset partial event stuff */
@@ -964,19 +1025,24 @@ midi_enter_smpte_mode(softc)
 }
 
 int
-midi_event2smf(softc, event, smf)
+midi_event2smf(softc, event, smf, raw)
 	struct midi_softc *softc;
 	struct event *event;
 	struct stynamic *smf;
+	int raw;
 {
-	long tempo, smf_ticks;
+	unsigned long smf_ticks;
+	long tempo;
 	int tmp_len;
 	u_char tmp_buf[4];
 
 	/* convert from timing ticks to SMF ticks */
-	smf_ticks = midi_timing2smf_tick(softc, event->tempo, event->time);
-	tmp_len = midi_fix2var(smf_ticks, tmp_buf);
-	stynamic_add_bytes(smf, tmp_buf, tmp_len);
+	if (!raw) {
+		smf_ticks = midi_timing2smf_tick(softc, event->tempo,
+		    event->time);
+		tmp_len = midi_fix2var(smf_ticks, tmp_buf);
+		stynamic_add_bytes(smf, tmp_buf, tmp_len);
+	}
 
 	switch (event->type) {
 	case NORMAL:
@@ -1039,37 +1105,45 @@ midi_next_byte(softc, uio)
 }
 
 int
-midi_uio2event(softc, uio, event)
+midi_uio2event(softc, uio, event, raw)
 	struct midi_softc *softc;
 	void *uio;
 	struct event *event;
+	int raw;
 {
 	u_long smf_ticks, ulen;
 	long tempo;
 	int byte, extra_byte, i, len, num_data_bytes;
 	int rs_change;
-	u_char meta_type, tmp_buf[256];
+	u_char meta_type, *rs, tmp_buf[256];
 #ifdef GJWDEBUG
 	struct SMPTE_frame tmp_frame;
 #endif
 
-	/* copy in timing portion */
-	len = 0;
-	do {
-		if ((byte = midi_next_byte(softc, uio)) == -1) {
-			stynamic_release(&event->data);
-			return (-1);
-		}
-		tmp_buf[len++] = byte;
-	} while (byte & 0x80);
+	if (raw) {
+		event->time = 0;
+		event->smf_time = 0;
+		rs = &softc->raw_writers;
+	} else {
+		rs = &softc->writers;
+		/* copy in timing portion */
+		len = 0;
+		do {
+			if ((byte = midi_next_byte(softc, uio)) == -1) {
+				stynamic_release(&event->data);
+				return (-1);
+			}
+			tmp_buf[len++] = byte;
+		} while (byte & 0x80);
 
-	/* compute time in smf ticks */
-	midi_var2fix(tmp_buf, &smf_ticks);
+		/* compute time in smf ticks */
+		midi_var2fix(tmp_buf, &smf_ticks);
 
-	/* now convert from smf to timing */
-	event->time = midi_smf2timing_tick(softc, smf_ticks);
-	softc->write_smf_time += smf_ticks;
-	event->smf_time = softc->write_smf_time;
+		/* now convert from smf to timing */
+		event->time = midi_smf2timing_tick(softc, smf_ticks);
+		softc->write_smf_time += smf_ticks;
+		event->smf_time = softc->write_smf_time;
+	}
 
 	/* get first byte of event data */
 	if ((byte = midi_next_byte(softc, uio)) == -1) {
@@ -1194,7 +1268,9 @@ midi_uio2event(softc, uio, event)
 		break;
 	default:
 		if ((byte & 0xf0) == 0x80) {
+#if 0
 			u_char rs_chan, rs_type;
+#endif
 
 			/* check note off events separately */
 			tmp_buf[0] = byte;
@@ -1214,8 +1290,8 @@ midi_uio2event(softc, uio, event)
 			 * check to see if we can collapse and use
 			 * running state
 			 */
-			rs_type = softc->writers & 0xf0;
-			rs_chan = softc->writers & 0x0f;
+			rs_type = *rs & 0xf0;
+			rs_chan = *rs & 0x0f;
 			/*
 			 * We are no longer using running status, as
 			 * it causes trouble when MIDI_THRU is on.
@@ -1227,23 +1303,23 @@ midi_uio2event(softc, uio, event)
 			 * same channel and the note off velocity is
 			 * zero.
 			 */
-			if (event->time == 0 && (softc->writers == tmp_buf[0]
+			if (event->time == 0 && (*rs == tmp_buf[0]
 			    || (tmp_buf[2] == 0 && rs_type == 0x90
 			    && rs_chan == (tmp_buf[0] & 0x0f)))) {
 				tmp_buf[0] = tmp_buf[1];
 				tmp_buf[1] = tmp_buf[2];
 				len = 2;
 			} else {
-				softc->writers = tmp_buf[0];
+				*rs = tmp_buf[0];
 			}
 #else
-			softc->writers = tmp_buf[0];
+			*rs = tmp_buf[0];
 #endif
 		} else {
 			extra_byte = 0;
 			rs_change = 0;
-			if ((byte & 0x80) && (byte != softc->writers)) {
-				softc->writers = byte;
+			if ((byte & 0x80) && (byte != *rs)) {
+				*rs = byte;
 				rs_change = 1;
 			}
 			len = 0;
@@ -1255,11 +1331,11 @@ midi_uio2event(softc, uio, event)
 				 * is turned on while we're playing
 				 * also add it if the running state changes
 				 */
-				tmp_buf[0] = softc->writers;
+				tmp_buf[0] = *rs;
 				len = 1;
 			}
 #else
-			tmp_buf[0] = softc->writers;
+			tmp_buf[0] = *rs;
 			len = 1;
 #endif
 			if (byte & 0x80)
@@ -1269,7 +1345,7 @@ midi_uio2event(softc, uio, event)
 				len++;
 			}
 
-			switch (softc->writers & 0xf0) {
+			switch (*rs & 0xf0) {
 			case 0x80:
 			case 0x90:
 			case 0xa0:
@@ -1408,8 +1484,11 @@ midi_add_complete_event(softc)
 		break;
 	}
 	stynamic_append(&event.data, &pe->event);
-	if (softc->status & MIDI_THRU)
-		midi_write_event(softc, &event, 0, 1);
+	if (softc->status & MIDI_THRU) {
+		/* use a 0 delay timeout to write the event */
+		midi_copy_event(&softc->write_event, &event);
+		TIMEOUT(&softc->write_timeout_id, midi_write_timeout, softc, 0);
+	}
 	/* enqueue new event */
 	if (softc->status & MIDI_READING)
 		midi_enq(softc->rqueue, &event);
@@ -1424,7 +1503,7 @@ midi_add_complete_event(softc)
 		softc->status &= ~MIDI_RD_SLEEP;
 		WAKEUP(&softc->sleep_read);
 	}
-	if (softc->status & MIDI_ASYNC)
+	if ((softc->status & MIDI_ASYNC) && (softc->status & MIDI_READING))
 		SIGNAL_PG(softc->pgid, SIGIO);
 	if (softc->status & MIDI_RSEL) {
 		softc->status &= ~MIDI_RSEL;
@@ -1486,7 +1565,7 @@ midi_timeout(arg)
 		else
 			quiet = 0;
 
-		midi_write_event(softc, event, quiet, 0);
+		midi_write_event(softc, event, quiet, 1);
 		softc->prev_outgoing += event->time;
 		stynamic_release(&event->data);
 		if (!midi_peekq(softc->wqueue, &event))
@@ -1502,6 +1581,16 @@ midi_timeout(arg)
 	return;
 }
 
+void
+midi_write_timeout(arg)
+	TIMEOUT_ARG arg;
+{
+	struct midi_softc *softc = (struct midi_softc *)arg;
+
+	midi_write_event(softc, &softc->write_event, 0, 0);
+	stynamic_release(&softc->write_event.data);
+}
+
 void
 midi_dropped_SMPTE_sync(arg)
 	TIMEOUT_ARG arg;
@@ -1533,14 +1622,14 @@ midi_dropped_SMPTE_sync(arg)
 }
 
 void
-midi_write_event(softc, event, quiet, midithru)
+midi_write_event(softc, event, quiet, notify)
 	struct midi_softc *softc;
 	struct event *event;
 	int quiet;
-	int midithru;
+	int notify;
 {
 	struct tempo_change *tchange, *prev_tchange;
-	int i, s;
+	int i;
 	u_char bytes[4], channel, command, *dataptr;
 
 	switch (event->type) {
@@ -1616,18 +1705,16 @@ midi_write_event(softc, event, quiet, midithru)
 		/* FALLTHRU */
 	default:
 		if (!quiet) {
-			s = BLOCK_INTR();
 			for (i = 0; i < event->data.len; i++) {
 				if (!midi_wait_rdy_rcv(softc))
 					break;
 				OUTB(softc->addr + MIDI_DATA,
 				    stynamic_get_byte(&event->data, i));
 			}
-			UNBLOCK_INTR(s);
 		}
 		break;
 	}
-	if (!midithru)
+	if (notify)
 		midi_send_sigio(softc);
 }
 
@@ -2397,7 +2484,7 @@ stynamic_get_byte(sty, index)
 	int index;
 {
 
-	if (sty->len <= 4)
+	if (sty->len <= STYNAMIC_SIZE)
 		return (sty->datas[index]);
 	else
 		return (sty->datad[index]);
@@ -2412,7 +2499,7 @@ stynamic_copy(sty, dest, len)
 	int s;
 
 	s = BLOCK_INTR();
-	if (sty->len <= 4)
+	if (sty->len <= STYNAMIC_SIZE)
 		BCOPY(sty->datas, dest, len);
 	else
 		BCOPY(sty->datad, dest, len);
@@ -2425,7 +2512,7 @@ stynamic_append(dest, src)
 	struct stynamic *src;
 {
 
-	if (src->len <= 4)
+	if (src->len <= STYNAMIC_SIZE)
 		stynamic_add_bytes(dest, src->datas, src->len);
 	else
 		stynamic_add_bytes(dest, src->datad, src->len);
@@ -2442,7 +2529,7 @@ stynamic_copy_from(sty, index, dest, len)
 	int s;
 
 	s = BLOCK_INTR();
-	if (sty->len <= 4)
+	if (sty->len <= STYNAMIC_SIZE)
 		BCOPY(&sty->datas[index], dest, len);
 	else
 		BCOPY(&sty->datad[index], dest, len);
