@@ -71,6 +71,8 @@ Event *
 EventTree::GetEvents(unsigned long time)
 {
 	EventTreeNode *n;
+	Event *e;
+	int valid_event;
 
 	n = rb_find_ukey(head, time);
 	if (n == head) {
@@ -79,6 +81,43 @@ EventTree::GetEvents(unsigned long time)
 	}
 
 	curr_event = (Event *)n->v.val;
+	/*
+	 * We need to check if the events we found are all the
+	 * NoteOff half of Note events.  If so we have to discard
+	 * it and find the next non-NoteOff event.
+	 */
+	valid_event = 0;
+	do {
+		/* check event type(s) */
+		for (e = curr_event; e != 0; e = e->GetNextEvent()) {
+			/*
+			 * It is a valid event if the following
+			 * conditions do *not* occur.
+			 * The event is a NoteOff event or a
+			 * NoteOn event with a velocity of 0,
+			 * and it has a NotePair.
+			 */
+			if (!((e->GetType() == NOTEOFF ||
+			    (e->GetType() == NOTEON &&
+			    ((NoteEvent *)e)->GetVelocity() == 0)) &&
+			    ((NoteEvent *)e)->GetNotePair() != 0)) {
+				valid_event = 1;
+				break;
+			}
+		}
+		/*
+		 * get the next node
+		 */
+		if (!valid_event) {
+			n = rb_next(n);
+			if (n != head->c.list.flink && n != head)
+				curr_event = (Event *)n->v.val;
+			else {
+				curr_event = 0;
+				return (0);
+			}
+		}
+	} while (!valid_event);
 	return (curr_event);
 }
 
