@@ -8,12 +8,12 @@ extern "C" {
 
 #include "tclmidi.h"
 
-#ifdef TCL_7_4
+#ifndef TCL_7_3
 int
 main(int argc, char **argv)
 {
 
-	Tcl_Main(argc, argv);
+	Tcl_Main(argc, argv, Tcl_AppInit);
 	return (0);
 }
 #endif
