@@ -56,7 +56,7 @@ Tclm_PrintEvent(ostream &buf, Event *e)
 		if (((NoteEvent *)e)->GetNotePair() == 0)
 			str = Tclm_PrintNoteOn((NoteOnEvent *)e);
 		else {
-			if (((NoteEvent *)e)->GetVelocity() == 0) {
+			if ((int)((NoteEvent *)e)->GetVelocity() == 0) {
 				buf << ends;
 				return;
 			}
@@ -252,7 +252,7 @@ Tclm_PrintMetaText(MetaTextEvent *e)
 {
 	ostrstream buf;
 
-	buf << "MetaText \"" << e->GetString() << "\"" << ends;
+	buf << "MetaText {" << e->GetString() << "}" << ends;
 	return (buf.str());
 }
 
@@ -261,7 +261,7 @@ Tclm_PrintMetaCopyright(MetaCopyrightEvent *e)
 {
 	ostrstream buf;
 
-	buf << "MetaCopyright \"" << e->GetString() << "\"" << ends;
+	buf << "MetaCopyright {" << e->GetString() << "}" << ends;
 	return (buf.str());
 }
 
@@ -270,7 +270,7 @@ Tclm_PrintMetaSequenceName(MetaSequenceNameEvent *e)
 {
 	ostrstream buf;
 
-	buf << "MetaSequenceName \"" << e->GetString() << "\"" << ends;
+	buf << "MetaSequenceName {" << e->GetString() << "}" << ends;
 	return (buf.str());
 }
 
@@ -279,7 +279,7 @@ Tclm_PrintMetaInstrumentName(MetaInstrumentNameEvent *e)
 {
 	ostrstream buf;
 
-	buf << "MetaInstrumentName \"" << e->GetString() << "\"" << ends;
+	buf << "MetaInstrumentName {" << e->GetString() << "}" << ends;
 	return (buf.str());
 }
 
@@ -288,7 +288,7 @@ Tclm_PrintMetaLyric(MetaLyricEvent *e)
 {
 	ostrstream buf;
 
-	buf << "MetaLyric \"" << e->GetString() << "\"" << ends;
+	buf << "MetaLyric {" << e->GetString() << "}" << ends;
 	return (buf.str());
 }
 
@@ -297,7 +297,7 @@ Tclm_PrintMetaMarker(MetaMarkerEvent *e)
 {
 	ostrstream buf;
 
-	buf << "MetaMarker \"" << e->GetString() << "\"" << ends;
+	buf << "MetaMarker {" << e->GetString() << "}" << ends;
 	return (buf.str());
 }
 
@@ -306,7 +306,7 @@ Tclm_PrintMetaCue(MetaCueEvent *e)
 {
 	ostrstream buf;
 
-	buf << "MetaCue \"" << e->GetString() << "\"" << ends;
+	buf << "MetaCue {" << e->GetString() << "}" << ends;
 	return (buf.str());
 }
 
@@ -376,7 +376,7 @@ Tclm_PrintMetaKey(MetaKeyEvent *e)
 {
 	ostrstream buf;
 
-	buf << "MetaKey \"" << e->GetKeyStr() << "\" " << e->GetModeStr()
+	buf << "MetaKey {" << e->GetKeyStr() << "} " << e->GetModeStr()
 	    << ends;
 	return (buf.str());
 }
