@@ -1,7 +1,7 @@
-#ifndef TCLM_H
-#define TCLM_H
+#ifndef TCLMIDI_H
+#define TCLMIDI_H
 
-#include "TclmInterp.h"
+#include "TclmIntp.h"
 
 extern int Tclm_Init(Tcl_Interp *interp);
 extern int Tclm_PlayInit(Tcl_Interp *interp, TclmInterp *tclm_interp);
