@@ -16,12 +16,12 @@ int *tclDummyMainPtr = (int *)TkMain;
 }
 #endif
 
-#ifdef TCL_7_4
+#ifndef TCL_7_3
 int
 main(int argc, char **argv)
 {
 
-	Tk_Main(argc, argv);
+	Tk_Main(argc, argv, Tcl_AppInit);
 	return (0);
 }
 #endif
